/*
 * DO NOT EDIT.  THIS FILE IS GENERATED FROM ../../../dist/idl/nsIIccProvider.idl
 */

#ifndef __gen_nsIIccProvider_h__
#define __gen_nsIIccProvider_h__


#ifndef __gen_nsISupports_h__
#include "nsISupports.h"
#endif

#include "js/Value.h"

/* For IDL files that don't want to include root IDL files. */
#ifndef NS_NO_VTABLE
#define NS_NO_VTABLE
#endif
class nsIDOMDOMRequest; /* forward declaration */

class nsIDOMWindow; /* forward declaration */

class nsIIccInfo; /* forward declaration */

class nsIIccListener; /* forward declaration */


/* starting interface:    nsIIccChannelCallback */
#define NS_IICCCHANNELCALLBACK_IID_STR "6136acab-b50e-494a-a86d-df392a032897"

#define NS_IICCCHANNELCALLBACK_IID \
  {0x6136acab, 0xb50e, 0x494a, \
    { 0xa8, 0x6d, 0xdf, 0x39, 0x2a, 0x03, 0x28, 0x97 }}

class NS_NO_VTABLE nsIIccChannelCallback : public nsISupports {
 public: 

  NS_DECLARE_STATIC_IID_ACCESSOR(NS_IICCCHANNELCALLBACK_IID)

  /* void notifyOpenChannelSuccess (in long channel); */
  NS_IMETHOD NotifyOpenChannelSuccess(int32_t channel) = 0;

  /* void notifyCloseChannelSuccess (); */
  NS_IMETHOD NotifyCloseChannelSuccess(void) = 0;

  /* void notifyExchangeAPDUResponse (in octet sw1, in octet sw2, in DOMString data); */
  NS_IMETHOD NotifyExchangeAPDUResponse(uint8_t sw1, uint8_t sw2, const nsAString & data) = 0;

  /* void notifyError (in DOMString error); */
  NS_IMETHOD NotifyError(const nsAString & error) = 0;

};

  NS_DEFINE_STATIC_IID_ACCESSOR(nsIIccChannelCallback, NS_IICCCHANNELCALLBACK_IID)

/* Use this macro when declaring classes that implement this interface. */
#define NS_DECL_NSIICCCHANNELCALLBACK \
  NS_IMETHOD NotifyOpenChannelSuccess(int32_t channel) override; \
  NS_IMETHOD NotifyCloseChannelSuccess(void) override; \
  NS_IMETHOD NotifyExchangeAPDUResponse(uint8_t sw1, uint8_t sw2, const nsAString & data) override; \
  NS_IMETHOD NotifyError(const nsAString & error) override; 

/* Use this macro to declare functions that forward the behavior of this interface to another object. */
#define NS_FORWARD_NSIICCCHANNELCALLBACK(_to) \
  NS_IMETHOD NotifyOpenChannelSuccess(int32_t channel) override { return _to NotifyOpenChannelSuccess(channel); } \
  NS_IMETHOD NotifyCloseChannelSuccess(void) override { return _to NotifyCloseChannelSuccess(); } \
  NS_IMETHOD NotifyExchangeAPDUResponse(uint8_t sw1, uint8_t sw2, const nsAString & data) override { return _to NotifyExchangeAPDUResponse(sw1, sw2, data); } \
  NS_IMETHOD NotifyError(const nsAString & error) override { return _to NotifyError(error); } 

/* Use this macro to declare functions that forward the behavior of this interface to another object in a safe way. */
#define NS_FORWARD_SAFE_NSIICCCHANNELCALLBACK(_to) \
  NS_IMETHOD NotifyOpenChannelSuccess(int32_t channel) override { return !_to ? NS_ERROR_NULL_POINTER : _to->NotifyOpenChannelSuccess(channel); } \
  NS_IMETHOD NotifyCloseChannelSuccess(void) override { return !_to ? NS_ERROR_NULL_POINTER : _to->NotifyCloseChannelSuccess(); } \
  NS_IMETHOD NotifyExchangeAPDUResponse(uint8_t sw1, uint8_t sw2, const nsAString & data) override { return !_to ? NS_ERROR_NULL_POINTER : _to->NotifyExchangeAPDUResponse(sw1, sw2, data); } \
  NS_IMETHOD NotifyError(const nsAString & error) override { return !_to ? NS_ERROR_NULL_POINTER : _to->NotifyError(error); } 

#if 0
/* Use the code below as a template for the implementation class for this interface. */

/* Header file */
class nsIccChannelCallback : public nsIIccChannelCallback
{
public:
  NS_DECL_ISUPPORTS
  NS_DECL_NSIICCCHANNELCALLBACK

  nsIccChannelCallback();

private:
  ~nsIccChannelCallback();

protected:
  /* additional members */
};

/* Implementation file */
NS_IMPL_ISUPPORTS(nsIccChannelCallback, nsIIccChannelCallback)

nsIccChannelCallback::nsIccChannelCallback()
{
  /* member initializers and constructor code */
}

nsIccChannelCallback::~nsIccChannelCallback()
{
  /* destructor code */
}

/* void notifyOpenChannelSuccess (in long channel); */
NS_IMETHODIMP nsIccChannelCallback::NotifyOpenChannelSuccess(int32_t channel)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* void notifyCloseChannelSuccess (); */
NS_IMETHODIMP nsIccChannelCallback::NotifyCloseChannelSuccess()
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* void notifyExchangeAPDUResponse (in octet sw1, in octet sw2, in DOMString data); */
NS_IMETHODIMP nsIccChannelCallback::NotifyExchangeAPDUResponse(uint8_t sw1, uint8_t sw2, const nsAString & data)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* void notifyError (in DOMString error); */
NS_IMETHODIMP nsIccChannelCallback::NotifyError(const nsAString & error)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* End of implementation class template. */
#endif


/* starting interface:    nsIIccProvider */
#define NS_IICCPROVIDER_IID_STR "7dd6e186-b007-11e4-9b7e-7717d7863cb8"

#define NS_IICCPROVIDER_IID \
  {0x7dd6e186, 0xb007, 0x11e4, \
    { 0x9b, 0x7e, 0x77, 0x17, 0xd7, 0x86, 0x3c, 0xb8 }}

class NS_NO_VTABLE nsIIccProvider : public nsISupports {
 public: 

  NS_DECLARE_STATIC_IID_ACCESSOR(NS_IICCPROVIDER_IID)

  /* void registerIccMsg (in unsigned long clientId, in nsIIccListener listener); */
  NS_IMETHOD RegisterIccMsg(uint32_t clientId, nsIIccListener *listener) = 0;

  /* void unregisterIccMsg (in unsigned long clientId, in nsIIccListener listener); */
  NS_IMETHOD UnregisterIccMsg(uint32_t clientId, nsIIccListener *listener) = 0;

  /* void sendStkResponse (in unsigned long clientId, in nsIDOMWindow window, in jsval command, in jsval response); */
  NS_IMETHOD SendStkResponse(uint32_t clientId, nsIDOMWindow *window, JS::HandleValue command, JS::HandleValue response) = 0;

  /* void sendStkMenuSelection (in unsigned long clientId, in nsIDOMWindow window, in unsigned short itemIdentifier, in boolean helpRequested); */
  NS_IMETHOD SendStkMenuSelection(uint32_t clientId, nsIDOMWindow *window, uint16_t itemIdentifier, bool helpRequested) = 0;

  /* void sendStkTimerExpiration (in unsigned long clientId, in nsIDOMWindow window, in jsval timer); */
  NS_IMETHOD SendStkTimerExpiration(uint32_t clientId, nsIDOMWindow *window, JS::HandleValue timer) = 0;

  /* void sendStkEventDownload (in unsigned long clientId, in nsIDOMWindow window, in jsval event); */
  NS_IMETHOD SendStkEventDownload(uint32_t clientId, nsIDOMWindow *window, JS::HandleValue event) = 0;

  /* nsIDOMDOMRequest readContacts (in unsigned long clientId, in nsIDOMWindow window, in unsigned long contactType); */
  NS_IMETHOD ReadContacts(uint32_t clientId, nsIDOMWindow *window, uint32_t contactType, nsIDOMDOMRequest * *_retval) = 0;

  /* nsIDOMDOMRequest updateContact (in unsigned long clientId, in nsIDOMWindow window, in unsigned long contactType, in jsval contact, in DOMString pin2); */
  NS_IMETHOD UpdateContact(uint32_t clientId, nsIDOMWindow *window, uint32_t contactType, JS::HandleValue contact, const nsAString & pin2, nsIDOMDOMRequest * *_retval) = 0;

  /* void iccOpenChannel (in unsigned long clientId, in DOMString aid, in nsIIccChannelCallback callback); */
  NS_IMETHOD IccOpenChannel(uint32_t clientId, const nsAString & aid, nsIIccChannelCallback *callback) = 0;

  /* void iccExchangeAPDU (in unsigned long clientId, in long channel, in octet cla, in octet ins, in octet p1, in octet p2, in short p3, in DOMString data, in nsIIccChannelCallback callback); */
  NS_IMETHOD IccExchangeAPDU(uint32_t clientId, int32_t channel, uint8_t cla, uint8_t ins, uint8_t p1, uint8_t p2, int16_t p3, const nsAString & data, nsIIccChannelCallback *callback) = 0;

  /* void iccCloseChannel (in unsigned long clientId, in long channel, in nsIIccChannelCallback callback); */
  NS_IMETHOD IccCloseChannel(uint32_t clientId, int32_t channel, nsIIccChannelCallback *callback) = 0;

};

  NS_DEFINE_STATIC_IID_ACCESSOR(nsIIccProvider, NS_IICCPROVIDER_IID)

/* Use this macro when declaring classes that implement this interface. */
#define NS_DECL_NSIICCPROVIDER \
  NS_IMETHOD RegisterIccMsg(uint32_t clientId, nsIIccListener *listener) override; \
  NS_IMETHOD UnregisterIccMsg(uint32_t clientId, nsIIccListener *listener) override; \
  NS_IMETHOD SendStkResponse(uint32_t clientId, nsIDOMWindow *window, JS::HandleValue command, JS::HandleValue response) override; \
  NS_IMETHOD SendStkMenuSelection(uint32_t clientId, nsIDOMWindow *window, uint16_t itemIdentifier, bool helpRequested) override; \
  NS_IMETHOD SendStkTimerExpiration(uint32_t clientId, nsIDOMWindow *window, JS::HandleValue timer) override; \
  NS_IMETHOD SendStkEventDownload(uint32_t clientId, nsIDOMWindow *window, JS::HandleValue event) override; \
  NS_IMETHOD ReadContacts(uint32_t clientId, nsIDOMWindow *window, uint32_t contactType, nsIDOMDOMRequest * *_retval) override; \
  NS_IMETHOD UpdateContact(uint32_t clientId, nsIDOMWindow *window, uint32_t contactType, JS::HandleValue contact, const nsAString & pin2, nsIDOMDOMRequest * *_retval) override; \
  NS_IMETHOD IccOpenChannel(uint32_t clientId, const nsAString & aid, nsIIccChannelCallback *callback) override; \
  NS_IMETHOD IccExchangeAPDU(uint32_t clientId, int32_t channel, uint8_t cla, uint8_t ins, uint8_t p1, uint8_t p2, int16_t p3, const nsAString & data, nsIIccChannelCallback *callback) override; \
  NS_IMETHOD IccCloseChannel(uint32_t clientId, int32_t channel, nsIIccChannelCallback *callback) override; 

/* Use this macro to declare functions that forward the behavior of this interface to another object. */
#define NS_FORWARD_NSIICCPROVIDER(_to) \
  NS_IMETHOD RegisterIccMsg(uint32_t clientId, nsIIccListener *listener) override { return _to RegisterIccMsg(clientId, listener); } \
  NS_IMETHOD UnregisterIccMsg(uint32_t clientId, nsIIccListener *listener) override { return _to UnregisterIccMsg(clientId, listener); } \
  NS_IMETHOD SendStkResponse(uint32_t clientId, nsIDOMWindow *window, JS::HandleValue command, JS::HandleValue response) override { return _to SendStkResponse(clientId, window, command, response); } \
  NS_IMETHOD SendStkMenuSelection(uint32_t clientId, nsIDOMWindow *window, uint16_t itemIdentifier, bool helpRequested) override { return _to SendStkMenuSelection(clientId, window, itemIdentifier, helpRequested); } \
  NS_IMETHOD SendStkTimerExpiration(uint32_t clientId, nsIDOMWindow *window, JS::HandleValue timer) override { return _to SendStkTimerExpiration(clientId, window, timer); } \
  NS_IMETHOD SendStkEventDownload(uint32_t clientId, nsIDOMWindow *window, JS::HandleValue event) override { return _to SendStkEventDownload(clientId, window, event); } \
  NS_IMETHOD ReadContacts(uint32_t clientId, nsIDOMWindow *window, uint32_t contactType, nsIDOMDOMRequest * *_retval) override { return _to ReadContacts(clientId, window, contactType, _retval); } \
  NS_IMETHOD UpdateContact(uint32_t clientId, nsIDOMWindow *window, uint32_t contactType, JS::HandleValue contact, const nsAString & pin2, nsIDOMDOMRequest * *_retval) override { return _to UpdateContact(clientId, window, contactType, contact, pin2, _retval); } \
  NS_IMETHOD IccOpenChannel(uint32_t clientId, const nsAString & aid, nsIIccChannelCallback *callback) override { return _to IccOpenChannel(clientId, aid, callback); } \
  NS_IMETHOD IccExchangeAPDU(uint32_t clientId, int32_t channel, uint8_t cla, uint8_t ins, uint8_t p1, uint8_t p2, int16_t p3, const nsAString & data, nsIIccChannelCallback *callback) override { return _to IccExchangeAPDU(clientId, channel, cla, ins, p1, p2, p3, data, callback); } \
  NS_IMETHOD IccCloseChannel(uint32_t clientId, int32_t channel, nsIIccChannelCallback *callback) override { return _to IccCloseChannel(clientId, channel, callback); } 

/* Use this macro to declare functions that forward the behavior of this interface to another object in a safe way. */
#define NS_FORWARD_SAFE_NSIICCPROVIDER(_to) \
  NS_IMETHOD RegisterIccMsg(uint32_t clientId, nsIIccListener *listener) override { return !_to ? NS_ERROR_NULL_POINTER : _to->RegisterIccMsg(clientId, listener); } \
  NS_IMETHOD UnregisterIccMsg(uint32_t clientId, nsIIccListener *listener) override { return !_to ? NS_ERROR_NULL_POINTER : _to->UnregisterIccMsg(clientId, listener); } \
  NS_IMETHOD SendStkResponse(uint32_t clientId, nsIDOMWindow *window, JS::HandleValue command, JS::HandleValue response) override { return !_to ? NS_ERROR_NULL_POINTER : _to->SendStkResponse(clientId, window, command, response); } \
  NS_IMETHOD SendStkMenuSelection(uint32_t clientId, nsIDOMWindow *window, uint16_t itemIdentifier, bool helpRequested) override { return !_to ? NS_ERROR_NULL_POINTER : _to->SendStkMenuSelection(clientId, window, itemIdentifier, helpRequested); } \
  NS_IMETHOD SendStkTimerExpiration(uint32_t clientId, nsIDOMWindow *window, JS::HandleValue timer) override { return !_to ? NS_ERROR_NULL_POINTER : _to->SendStkTimerExpiration(clientId, window, timer); } \
  NS_IMETHOD SendStkEventDownload(uint32_t clientId, nsIDOMWindow *window, JS::HandleValue event) override { return !_to ? NS_ERROR_NULL_POINTER : _to->SendStkEventDownload(clientId, window, event); } \
  NS_IMETHOD ReadContacts(uint32_t clientId, nsIDOMWindow *window, uint32_t contactType, nsIDOMDOMRequest * *_retval) override { return !_to ? NS_ERROR_NULL_POINTER : _to->ReadContacts(clientId, window, contactType, _retval); } \
  NS_IMETHOD UpdateContact(uint32_t clientId, nsIDOMWindow *window, uint32_t contactType, JS::HandleValue contact, const nsAString & pin2, nsIDOMDOMRequest * *_retval) override { return !_to ? NS_ERROR_NULL_POINTER : _to->UpdateContact(clientId, window, contactType, contact, pin2, _retval); } \
  NS_IMETHOD IccOpenChannel(uint32_t clientId, const nsAString & aid, nsIIccChannelCallback *callback) override { return !_to ? NS_ERROR_NULL_POINTER : _to->IccOpenChannel(clientId, aid, callback); } \
  NS_IMETHOD IccExchangeAPDU(uint32_t clientId, int32_t channel, uint8_t cla, uint8_t ins, uint8_t p1, uint8_t p2, int16_t p3, const nsAString & data, nsIIccChannelCallback *callback) override { return !_to ? NS_ERROR_NULL_POINTER : _to->IccExchangeAPDU(clientId, channel, cla, ins, p1, p2, p3, data, callback); } \
  NS_IMETHOD IccCloseChannel(uint32_t clientId, int32_t channel, nsIIccChannelCallback *callback) override { return !_to ? NS_ERROR_NULL_POINTER : _to->IccCloseChannel(clientId, channel, callback); } 

#if 0
/* Use the code below as a template for the implementation class for this interface. */

/* Header file */
class nsIccProvider : public nsIIccProvider
{
public:
  NS_DECL_ISUPPORTS
  NS_DECL_NSIICCPROVIDER

  nsIccProvider();

private:
  ~nsIccProvider();

protected:
  /* additional members */
};

/* Implementation file */
NS_IMPL_ISUPPORTS(nsIccProvider, nsIIccProvider)

nsIccProvider::nsIccProvider()
{
  /* member initializers and constructor code */
}

nsIccProvider::~nsIccProvider()
{
  /* destructor code */
}

/* void registerIccMsg (in unsigned long clientId, in nsIIccListener listener); */
NS_IMETHODIMP nsIccProvider::RegisterIccMsg(uint32_t clientId, nsIIccListener *listener)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* void unregisterIccMsg (in unsigned long clientId, in nsIIccListener listener); */
NS_IMETHODIMP nsIccProvider::UnregisterIccMsg(uint32_t clientId, nsIIccListener *listener)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* void sendStkResponse (in unsigned long clientId, in nsIDOMWindow window, in jsval command, in jsval response); */
NS_IMETHODIMP nsIccProvider::SendStkResponse(uint32_t clientId, nsIDOMWindow *window, JS::HandleValue command, JS::HandleValue response)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* void sendStkMenuSelection (in unsigned long clientId, in nsIDOMWindow window, in unsigned short itemIdentifier, in boolean helpRequested); */
NS_IMETHODIMP nsIccProvider::SendStkMenuSelection(uint32_t clientId, nsIDOMWindow *window, uint16_t itemIdentifier, bool helpRequested)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* void sendStkTimerExpiration (in unsigned long clientId, in nsIDOMWindow window, in jsval timer); */
NS_IMETHODIMP nsIccProvider::SendStkTimerExpiration(uint32_t clientId, nsIDOMWindow *window, JS::HandleValue timer)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* void sendStkEventDownload (in unsigned long clientId, in nsIDOMWindow window, in jsval event); */
NS_IMETHODIMP nsIccProvider::SendStkEventDownload(uint32_t clientId, nsIDOMWindow *window, JS::HandleValue event)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* nsIDOMDOMRequest readContacts (in unsigned long clientId, in nsIDOMWindow window, in unsigned long contactType); */
NS_IMETHODIMP nsIccProvider::ReadContacts(uint32_t clientId, nsIDOMWindow *window, uint32_t contactType, nsIDOMDOMRequest * *_retval)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* nsIDOMDOMRequest updateContact (in unsigned long clientId, in nsIDOMWindow window, in unsigned long contactType, in jsval contact, in DOMString pin2); */
NS_IMETHODIMP nsIccProvider::UpdateContact(uint32_t clientId, nsIDOMWindow *window, uint32_t contactType, JS::HandleValue contact, const nsAString & pin2, nsIDOMDOMRequest * *_retval)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* void iccOpenChannel (in unsigned long clientId, in DOMString aid, in nsIIccChannelCallback callback); */
NS_IMETHODIMP nsIccProvider::IccOpenChannel(uint32_t clientId, const nsAString & aid, nsIIccChannelCallback *callback)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* void iccExchangeAPDU (in unsigned long clientId, in long channel, in octet cla, in octet ins, in octet p1, in octet p2, in short p3, in DOMString data, in nsIIccChannelCallback callback); */
NS_IMETHODIMP nsIccProvider::IccExchangeAPDU(uint32_t clientId, int32_t channel, uint8_t cla, uint8_t ins, uint8_t p1, uint8_t p2, int16_t p3, const nsAString & data, nsIIccChannelCallback *callback)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* void iccCloseChannel (in unsigned long clientId, in long channel, in nsIIccChannelCallback callback); */
NS_IMETHODIMP nsIccProvider::IccCloseChannel(uint32_t clientId, int32_t channel, nsIIccChannelCallback *callback)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* End of implementation class template. */
#endif


#endif /* __gen_nsIIccProvider_h__ */
