/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"

interface imgIRequest;

[scriptable, uuid(10be55b3-2029-41a7-a975-538efed250ed)]
interface imgIScriptedNotificationObserver : nsISupports
{
  void sizeAvailable(in imgIRequest aRequest);
  void frameUpdate(in imgIRequest aRequest);
  void frameComplete(in imgIRequest aRequest);
  void loadComplete(in imgIRequest aRequest);
  void decodeComplete(in imgIRequest aRequest);
  void discard(in imgIRequest aRequest);
  void isAnimated(in imgIRequest aRequest);
  void hasTransparency(in imgIRequest aRequest);
};
