/* List of defines generated by configure. Included with preprocessor flag,
 * -include, to avoid long list of -D defines on the compile command-line.
 * Do not edit.
 */

#ifndef MOZILLA_CONFIG_H
#define MOZILLA_CONFIG_H

#if defined(__clang__)
#pragma clang diagnostic push
#if __has_warning("-Wreserved-id-macro")
#pragma clang diagnostic ignored "-Wreserved-id-macro"
#endif
#endif

#define A11Y_LOG 1
#define ACCESSIBILITY 1
#define ATK_MAJOR_VERSION 1
#define ATK_MINOR_VERSION 28
#define ATK_REV_VERSION 0
#define ATTRIBUTE_ALIGNED_MAX 64
#define BUILD_CTYPES 1
#define D_INO d_ino
#define E10S_TESTING_ONLY 1
#define EARLY_BETA_OR_EARLIER 1
#define ENABLE_INTL_API 1
#define ENABLE_MARIONETTE 1
#define ENABLE_SYSTEM_EXTENSION_DIRS 1
#define EXPOSE_INTL_API 1
#define FIREFOX_VERSION 40.0a2
#define FORCE_PR_LOG 1
#define FUNCPROTO 15
#define GLIB_VERSION_MAX_ALLOWED GLIB_VERSION_2_26
#define GLIB_VERSION_MIN_REQUIRED GLIB_VERSION_2_26
#define GL_PROVIDER_GLX 1
#define GST_API_VERSION "0.10"
#define HAVE_ALLOCA_H 1
#define HAVE_BYTESWAP_H 1
#define HAVE_CLOCK_MONOTONIC 1
#define HAVE_CPUID_H 1
#define HAVE_DIRENT_H 1
#define HAVE_DLADDR 1
#define HAVE_DLOPEN 1
#define HAVE_FONTCONFIG_FCFREETYPE_H 1
#define HAVE_FT_BITMAP_SIZE_Y_PPEM 1
#define HAVE_FT_GLYPHSLOT_EMBOLDEN 1
#define HAVE_FT_LOAD_SFNT_TABLE 1
#define HAVE_GETOPT_H 1
#define HAVE_GMTIME_R 1
#define HAVE_GNU_LIBC_VERSION_H 1
#define HAVE_I18N_LC_MESSAGES 1
#define HAVE_INTTYPES_H 1
#define HAVE_LANGINFO_CODESET 1
#define HAVE_LINUX_IF_ADDR_H 1
#define HAVE_LINUX_QUOTA_H 1
#define HAVE_LINUX_RTNETLINK_H 1
#define HAVE_LOCALECONV 1
#define HAVE_LOCALTIME_R 1
#define HAVE_LSTAT64 1
#define HAVE_MALLOC_H 1
#define HAVE_MALLOC_H 1
#define HAVE_MALLOC_USABLE_SIZE 1
#define HAVE_MEMALIGN 1
#define HAVE_MEMMEM 1
#define HAVE_MEMORY_H 1
#define HAVE_NETINET_IN_H 1
#define HAVE_NL_TYPES_H 1
#define HAVE_POSIX_FADVISE 1
#define HAVE_POSIX_FALLOCATE 1
#define HAVE_POSIX_MEMALIGN 1
#define HAVE_RES_NINIT 1
#define HAVE_STAT64 1
#define HAVE_STDINT_H 1
#define HAVE_STRNDUP 1
#define HAVE_SYS_BITYPES_H 1
#define HAVE_SYS_CDEFS_H 1
#define HAVE_SYS_QUEUE_H 1
#define HAVE_SYS_QUOTA_H 1
#define HAVE_SYS_SYSMACROS_H 1
#define HAVE_SYS_TYPES_H 1
#define HAVE_THREAD_TLS_KEYWORD 1
#define HAVE_TRUNCATE64 1
#define HAVE_UNISTD_H 1
#define HAVE_VALLOC 1
#define HAVE_VA_COPY 1
#define HAVE_VISIBILITY_ATTRIBUTE 1
#define HAVE_VISIBILITY_HIDDEN_ATTRIBUTE 1
#define HAVE_X11_XKBLIB_H 1
#define HAVE__UNWIND_BACKTRACE 1
#define HAVE___CXA_DEMANGLE 1
#define JS_DEFAULT_JITREPORT_GRANULARITY 3
#define LIBJPEG_TURBO_X86_ASM 1
#define MALLOC_H <malloc.h>
#define MALLOC_USABLE_SIZE_CONST_PTR 
#define MOZILLA_UAVERSION "40.0"
#define MOZILLA_VERSION "40.0a2"
#define MOZILLA_VERSION_U 40.0a2
#define MOZ_ACCESSIBILITY_ATK 1
#define MOZ_APP_UA_NAME ""
#define MOZ_APP_UA_VERSION "40.0a2"
#define MOZ_B2G_OS_NAME ""
#define MOZ_B2G_VERSION "1.0.0"
#define MOZ_BUILD_APP xulrunner
#define MOZ_CRASHREPORTER 1
#define MOZ_CRASHREPORTER_ENABLE_PERCENT 100
#define MOZ_DATA_REPORTING 1
#define MOZ_DEBUG_SYMBOLS 1
#define MOZ_DISTRIBUTION_ID "org.mozilla"
#define MOZ_DLL_SUFFIX ".so"
#define MOZ_EME 1
#define MOZ_ENABLE_DBUS 1
#define MOZ_ENABLE_GCONF 1
#define MOZ_ENABLE_GIO 1
#define MOZ_ENABLE_PROFILER_SPS 1
#define MOZ_ENABLE_SKIA 1
#define MOZ_ENABLE_XREMOTE 1
#define MOZ_FEEDS 1
#define MOZ_FFMPEG 1
#define MOZ_FMP4 1
#define MOZ_GAMEPAD 1
#define MOZ_GMP_SANDBOX 1
#define MOZ_GSTREAMER 1
#define MOZ_INSTRUMENT_EVENT_LOOP 1
#define MOZ_LOGGING 1
#define MOZ_MACBUNDLE_ID org.mozilla.xulrunner
#define MOZ_MEDIA_NAVIGATOR 1
#define MOZ_MEMORY 1
#define MOZ_MEMORY_LINUX 1
#define MOZ_OMNIJAR 1
#define MOZ_PDF_PRINTING 1
#define MOZ_PEERCONNECTION 1
#define MOZ_PERMISSIONS 1
#define MOZ_PLACES 1
#define MOZ_PROFILELOCKING 1
#define MOZ_RAW 1
#define MOZ_SAMPLE_TYPE_FLOAT32 1
#define MOZ_SANDBOX 1
#define MOZ_SCTP 1
#define MOZ_SECUREELEMENT 1
#define MOZ_SERVICES_COMMON 1
#define MOZ_SERVICES_CRYPTO 1
#define MOZ_SERVICES_HEALTHREPORT 1
#define MOZ_SERVICES_METRICS 1
#define MOZ_SERVICES_SYNC 1
#define MOZ_SIGNING 1
#define MOZ_SOCIAL 1
#define MOZ_SRTP 1
#define MOZ_STACKWALKING 1
#define MOZ_STATIC_JS 1
#define MOZ_TELEMETRY_DISPLAY_REV 2
#define MOZ_TREE_CAIRO 1
#define MOZ_TREE_PIXMAN 1
#define MOZ_UPDATER 1
#define MOZ_UPDATE_CHANNEL default
#define MOZ_URL_CLASSIFIER 1
#define MOZ_USER_DIR ".mozilla"
#define MOZ_VORBIS 1
#define MOZ_VPX 1
#define MOZ_VPX_ERROR_CONCEALMENT 1
#define MOZ_WAVE 1
#define MOZ_WEBM 1
#define MOZ_WEBRTC 1
#define MOZ_WEBRTC_ASSERT_ALWAYS 1
#define MOZ_WEBRTC_SIGNALING 1
#define MOZ_WEBSPEECH 1
#define MOZ_WIDGET_GTK 2
#define MOZ_WIDGET_GTK2 1
#define MOZ_X11 1
#define MOZ_XUL 1
#define MOZ_XULRUNNER 1
#define NS_PRINTING 1
#define NS_PRINT_PREVIEW 1
#define STDC_HEADERS 1
#define TARGET_XPCOM_ABI "x86-gcc3"
#define USE_SKIA 1
#define USE_SKIA_GPU 1
#define U_STATIC_IMPLEMENTATION 1
#define U_USING_ICU_NAMESPACE 0
#define VA_COPY va_copy
#define VPX_X86_ASM 1
#define XP_LINUX 1
#define XP_UNIX 1
#define _REENTRANT 1

/*
 * The c99 defining the limit macros (UINT32_MAX for example), says:
 *
 *   C++ implementations should define these macros only when
 *   __STDC_LIMIT_MACROS is defined before <stdint.h> is included.
 *
 * The same also occurs with __STDC_CONSTANT_MACROS for the constant macros
 * (INT8_C for example) used to specify a literal constant of the proper type,
 * and with __STDC_FORMAT_MACROS for the format macros (PRId32 for example) used
 * with the fprintf function family.
 */
#define __STDC_LIMIT_MACROS
#define __STDC_CONSTANT_MACROS
#if !defined(__STDC_FORMAT_MACROS)
#define __STDC_FORMAT_MACROS
#endif

#if defined(__clang__)
#pragma clang diagnostic pop
#endif

/*
 * Force-include Char16.h in order to define PRUnichar as char16_t everywhere.
 * Note that this should be the first #include to make sure that prtypes.h does
 * not attempt to define PRUnichar.  This includes the following hunspell-specific
 * includes.
 *
 * We don't use this to build elfhack and elf-dynstr-gc since those builds happen
 * during the export tier.  Also, disable this when building assembly files too.
 */
#if !defined(ELFHACK_BUILD) && !defined(__ASSEMBLER__)
#include "mozilla/Char16.h"
#endif

/*
 * Force-include hunspell_alloc_hooks.h and hunspell_fopen_hooks.h for hunspell,
 * so that we don't need to modify them directly.
 *
 * HUNSPELL_STATIC is defined in extensions/spellcheck/hunspell/src/Makefile.in,
 * unless --enable-system-hunspell is defined.
 */
#if defined(HUNSPELL_STATIC)
#include "hunspell_alloc_hooks.h"
#include "hunspell_fopen_hooks.h"
#endif

/*
 * Force-include sdkdecls.h for building the chromium sandbox code.
 *
 * CHROMIUM_SANDBOX_BUILD is defined in security/sandbox/moz.build.
 * Note that this include path relies on the LOCAL_INCLUDES in that file.
 */
#if defined(CHROMIUM_SANDBOX_BUILD) && defined(XP_WIN)
#include "base/win/sdkdecls.h"
#endif

#endif /* MOZILLA_CONFIG_H */
