/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */


#include "domstubs.idl"

[scriptable, uuid(b19038ba-0d75-40d2-be35-742e26d33bf9)]
interface nsITabParent : nsISupports
{
  void injectTouchEvent(in AString aType,
                        [array, size_is(count)] in uint32_t aIdentifiers,
                        [array, size_is(count)] in int32_t aXs,
                        [array, size_is(count)] in int32_t aYs,
                        [array, size_is(count)] in uint32_t aRxs,
                        [array, size_is(count)] in uint32_t aRys,
                        [array, size_is(count)] in float aRotationAngles,
                        [array, size_is(count)] in float aForces,
                        in uint32_t count,
                        in long aModifiers);

  void getChildProcessOffset(out int32_t aCssX, out int32_t aCssY);

  readonly attribute boolean useAsyncPanZoom;

  void setIsDocShellActive(in bool aIsActive);

  readonly attribute uint64_t tabId;
};
